<?php

namespace MakeCommerce\Shipping;

/**
 * All functionality that has to do with shipping labels and printing
 * 
 * @since 3.0.0
 */

class Label extends \MakeCommerce\Shipping {

	private $loader;
	
	//page format used for creating labels
	public $page_format = "A4";

	/**
	 * Constructs Label class, defines loader and hooks
	 * 
	 * @since 3.0.0
	 */
    public function __construct( \MakeCommerce\Loader $loader ) {
    
		$this->loader = $loader;

		$this->set_print_page_format();
		
		$this->define_hooks();
	}

	/**
	 * Set label print page format
	 * 
	 * @since 3.0.4
	 */
	public function set_print_page_format() {

		$this->page_format = get_option( 'mk_label_format', 'A4' );
	}
	
	/**
	 * Define all wordpress hooks needed for printing stuff and creating labels
	 * 
	 * @since 3.0.0
	 */
    public function define_hooks() {

        $this->loader->add_action( 'woocommerce_order_actions_end', $this, 'print_button' );
		$this->loader->add_action( 'wp_ajax_print_pml', $this, 'print' );
		$this->loader->add_filter( 'admin_action_parcel_machine_print_labels', $this, 'bulk_print' );
		$this->loader->add_filter( 'admin_footer', $this, 'bulk_print_and_register' );
		$this->loader->add_filter( 'admin_action_parcel_machine_labels', $this, 'bulk_register' );
    }

	/**
	 * Creates shipping label using MakeCommerce API and redirect to the page where it is available or return data
	 * 
	 * @since 3.0.0
	 */
    public function create_labels( $post_ids, $ajax = false ) {

		if ( !is_array( $post_ids ) ) {
			$post_ids = array( $post_ids );
		}
        
        $this->initalize();
        
		$shipping_request = array( 'credentials' => array(), 'orders' => array(), 'printFormat' => $this->page_format );
		
		foreach ( $this->shipping_methods as $shipping_method ) {
			$shipping_classes_map[$shipping_method["method"]] = $shipping_method["class"];
		}
        
        $missing_ids = array();
        
		foreach ( $post_ids as $post_id ) {

            $shipment_id = get_post_meta( $post_id, '_parcel_machine_shipment_id', true );
            
			if ( !$shipment_id ) { 
				$missing_ids[] = $post_id;
				continue;
            }
            
			$order = new \WC_Order( $post_id );
            $shipping_methods = $order->get_shipping_methods();
            
            if ( empty( $shipping_methods ) ) {
				continue;
			}
            
			foreach ( $shipping_methods as $shipping_method ) {

				$shipping_id = explode( ':', $shipping_method['method_id'] );
				$shipping_class = $shipping_id[0];
                $shipping_instance = !empty( $shipping_id[1] ) ? $shipping_id[1] : null;
                
				if ( empty( $shipping_classes_map[$shipping_class] ) ) {
					continue;
				}

				$transport_class = new $shipping_classes_map[$shipping_class]($shipping_instance);
				$carrier_uc = mb_strtoupper( $transport_class->carrier );
				
				//apt == parcelmachine
				if ( $transport_class->type === 'apt' ) {

					$parcel_machine = get_post_meta( $post_id, '_parcel_machine', true );

					if ( !$parcel_machine ) {
						continue;
					}

					list( $carrier, $machine_id ) = explode( '||', $parcel_machine );

					if ( !$carrier || !$machine_id ) {
						continue;
					}
                }
                
				if ( empty( $shipping_request['credentials'][$carrier_uc] ) ) {

					$api_user = $transport_class->settings['service_user'];
					$api_password = $transport_class->settings['service_password'];

					$use_mk_contract = false;

					if ( isset( $transport_class->settings['use_mk_contract'] ) ) {
						$use_mk_contract = $transport_class->settings['use_mk_contract'];
					}

					//change carrier_uc if it has been set in the interface.
					if ( isset( $transport_class->settings['service_carrier'] ) ) {
						$carrier_uc = $transport_class->settings['service_carrier'];
					}

					if ( ( !$api_user || !$api_password ) and ( !$use_mk_contract ) ) {

						$this->loader->add_action( 'admin_notices', $this, 'mk_admin_error' );
						continue;
					}

					if ( !$use_mk_contract ) {
						$shipping_request['credentials'][$carrier_uc] = array( 'carrier' => $carrier_uc, 'username' => $api_user, 'password' => $api_password );
					}
                }
                
				$sender = array(
					'name' => $transport_class->settings['shop_name'],
					'phone' => $transport_class->settings['shop_phone'],
					'email' => $transport_class->settings['shop_email'],
					'postalCode' => !empty( $transport_class->settings['shop_postal_code'] ) ? $transport_class->settings['shop_postal_code'] : '',
					'country' => !empty( $transport_class->settings['shop_address_country'] ) ? $transport_class->settings['shop_address_country'] : '',
					'city' => !empty( $transport_class->settings['shop_address_city'] ) ? $transport_class->settings['shop_address_city'] : '',
					'street' => !empty( $transport_class->settings['shop_address_street'] ) ? $transport_class->settings['shop_address_street'] : ''
                );
                
				$shipping_information = $this->get_order_shipping_information( $order );
                
				$sr_order = array(
					'carrier' => $carrier_uc,
					'orderId' => $order->get_id(),
					'recipient' => array( 'name' => $shipping_information['recipient_name'], 'phone' => $shipping_information['phone'], 'email' => $shipping_information['email'] ),
					'sender' => $sender,
					'shipmentId' => $shipment_id
                );
                
				$m_service_type = null;

				if ( $transport_class->carrier === 'omniva' && $transport_class->type === 'atp' ) {
					$m_service_type = 'PA';
				} else if ( !empty( $transport_class->settings['registerOnPaymentCode'] ) ) {
					$m_service_type = $transport_class->settings['registerOnPaymentCode'];
				}

				if ( $carrier_uc === 'LP_EXPRESS_LT' ) {
					
					$identifier = get_post_meta( $post_id, '_lp_express_cart_identifier', true );
					$sr_order['lpExpressShipmentDetails']['lpExpressCartIdentifier'] = $identifier;
				}
                
				if ( $transport_class->type === 'cou' || $carrier_uc === 'LP_EXPRESS_LT' ) {

					$sr_order['destination'] = array(
						'postalCode' => $order->shipping_postcode,
						'country' => $order->shipping_country,
						'county' => $order->shipping_state,
						'city' => $order->shipping_city,
						'street' => $order->shipping_address_1 . ' ' . $order->shipping_address_2
					);

					if ( $shipping_class == "courier_smartpost" ) {
						
						$delivery_time = get_post_meta( $post_id, '_delivery_time', true );
						
						if ( $delivery_time ) {
							$sr_order['destination']['timeWindow'] = $delivery_time;
						}
					}
				}

				if ( !empty( $machine_id ) ) {
					$sr_order['destination']['destinationId'] = $machine_id;
                }

				if ( $m_service_type ) {
					$sr_order['services'] = array( 'serviceType' => $m_service_type );
				}

				$shipping_request['orders'][] = $sr_order;
			}
        }
        
        $shipping_request['credentials'] = array_values( $shipping_request['credentials'] );
        
		if ( empty( $shipping_request['orders'] ) ) {
			return;
        }
        
        $MK = \MakeCommerce::get_api();
        
		if ( !$MK ) {
			return;
        }
        
		try {
			$response = $MK->createLabels( $shipping_request );
		} catch ( \Exception $e ) {

			echo $e->getMessage();
			exit();
        }
        
		if ( empty( $response->labelUrl ) ) {
			return;
        }
        
		if ( $ajax ) {
			return $response->labelUrl;
        }
        
		$mk_err = false;
		if ( !empty( $missing_ids ) ) {
            $mk_err = sprintf( __( 'Orders %s did not have shipment ID attached so no labels was printed! Please register those packages if you think this is an error!', 'wc_makecommerce_domain' ), join( ', ', $missing_ids ) );
        }

		$sendback = add_query_arg( array( 'post_type' => 'shop_order', 'mk_pdf' => urlencode( $response->labelUrl ), 'mk_err' => urlencode( $mk_err ) ), '' );
        
        wp_redirect( esc_url_raw( $sendback ) );
        
        exit();
	}

	/**
	 * Prints labels using ajax function in admin
	 * 
	 * @since 3.0.0
	 */
    public function print() {

		if ( !current_user_can( 'manage_woocommerce' ) ) {
			die();
        }
        
        echo $this->create_labels( array( intval( $_POST['id'] ) ), true );
        
		die();
	}
	
	/**
	 * Prints labels for all selected orders
	 * 
	 * @since 3.0.0
	 */
	public function bulk_print() {

		$post_ids = array_map( 'absint', ( array )$_REQUEST['post'] );

		$this->create_labels( $post_ids );
	}

	/**
	 * Adds bulk shipment register and printing actions to order view
	 * 
	 * @since 3.0.0
	 */
	public function bulk_print_and_register() {

		global $post_type;
		
		//Should this even be done using JS???
		if ( 'shop_order' === $post_type ) {
			
			$args = [
				'shipments_text' => __( 'Register parcel machine shipments', 'wc_makecommerce_domain' ), 
				'labels_text' => __( 'Print parcel machine labels', 'wc_makecommerce_domain' )
			];

			if ( !empty( $_REQUEST['mk_err'] ) ) {
				$args["error"] = htmlspecialchars( $_REQUEST['mk_err'] );
			}

			if ( !empty( $_REQUEST['mk_pdf'] ) ) {
				$args["pdf"] = $_REQUEST['mk_pdf'];
			}

			\MakeCommerce::mc_enqueue_script( 
				'MC_LABEL_BULK_ACTIONS', 
				dirname( __FILE__ ) . '/js/label_bulk_actions.js', 
				$args, 
				[ 'jquery' ]
			);
		}
    }

	/**
	 * Registers shipments in bulk (admin action)
	 * 
	 * @since 3.0.0
	 */
	public function bulk_register() {

		$post_ids = array_map( 'absint', ( array )$_REQUEST['post'] );

		if ( !empty($post_ids) ) {
			$this->register_shipment( $post_ids );
		}
	}

	/**
	 * Supporting function for print_labels. Creates needed javasript for printing button
	 * 
	 * @since 3.0.0
	 */
    public function print_button( $post_id ) {

        if ( !get_post_meta( $post_id, '_parcel_machine_shipment_id', true ) ) {
            return;
        }

	    echo '
	    <li class="wide">
		    <a id="print_parcel_machine_label" href="#" class="button">'. __( 'Print parcel label', 'wc_makecommerce_domain' ) .'</a>
		    <img class="mc_loading" src="' . site_url( '/wp-admin/images/loading.gif' ).'">
		</li>';

		\MakeCommerce::mc_enqueue_script( 
			'MC_LABEL_BUTTON', 
			dirname( __FILE__ ) . '/js/print_label.js', 
			[ 
				'post_id' => $post_id, 
				'error' => __( 'Something went wrong generating a label for this order. Please try again! Contact us, if the problem persists.', 'wc_makecommerce_domain' )
			], 
			[ 'jquery' ]
		);
    }
}