<?php

namespace MakeCommerce;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://makecommerce.net/
 * @since      3.0.0
 *
 * @package    Makecommerce
 * @subpackage Makecommerce/shipping
 */

/**
 * The shipping-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Makecommerce
 * @subpackage Makecommerce/shipping
 * @author     Maksekeskus AS <support@maksekeskus.ee>
 */
class Shipping {

	/**
	 * The ID of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
    private $version;
    
	private $loader;

	private Shipping\Label $label;
	private Shipping\Product $product;
	private Shipping\Order $order;
	
	/**
	 * Define all shipping methods
	 */
	public $shipping_methods = array(
		array(
			"option" => "mk_transport_apt_omniva",
			"method" => "parcelmachine_omniva",
			"class"  => "MakeCommerce\Shipping\Method\ParcelMachine\Omniva"
		),
		array(
			"option" => "mk_transport_apt_smartpost",
			"method" => "parcelmachine_smartpost",
			"class"  => "MakeCommerce\Shipping\Method\ParcelMachine\Smartpost"
		),
		array(
			"option" => "mk_transport_apt_dpd",
			"method" => "parcelmachine_dpd",
			"class"  => "MakeCommerce\Shipping\Method\ParcelMachine\DPD"
		),
		array(
			"option" => "mk_transport_apt_lp_express_lt",
			"default_name" => "LP Express",
			"method" => "parcelmachine_lp_express_lt",
			"class"  => "MakeCommerce\Shipping\Method\ParcelMachine\LPExpress"
		),
		array(
			"option" => "mk_transport_courier_omniva",
			"method" => "courier_omniva",
			"class"  => "MakeCommerce\Shipping\Method\Courier\Omniva"
		),
		array(
			"option" => "mk_transport_courier_smartpost",
			"method" => "courier_smartpost",
			"class"  => "MakeCommerce\Shipping\Method\Courier\Smartpost"
		)
	);

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, Loader $loader ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->loader = $loader;

		//loads all child classes such as label, admin specific functions, email functions
		$this->load_children();
		
		if ( is_admin() ) {
		    add_action( 'wp_ajax_verify_feature_swc', array( $this, 'verify_makecommerce_shipment_mediaton_availablity' ) );
		}
	}
	
	/**
     * Check if the customer has shipment mediation for Omniva enabled in MakeCommerce
     * 
     * @since 3.0.0
     */
    public function verify_makecommerce_shipment_mediaton_availablity() {

        $status = $this->check_shipment_mediaton_availablity( 'shipments_without_credentials' );

        wp_send_json( array( 'feature_name' => 'shipments_without_credentials', 'feature_status' => $status ) );

        exit; 
    }

    /**
     * Check if a shipment mediation is enabled in MakeCommerce
     * 
     * @since 3.0.0
     */
    public function check_shipment_mediaton_availablity( $feature_name = '' ) {
        
        $feature_enabled = false;
        
        $MK = \MakeCommerce::get_api();

        if ( !$MK ) {
            return false;
        }
                                
        try {
            $shopConfig = $MK->getShopConfig( \MakeCommerce::config_request_parameters( "makecommerce feature_staus_check" ) );
			$features = $shopConfig->features;	
			
        } catch ( \Exception $e ) {

            error_log( print_r( $e, 1 ) );

            return false;
        }
        
        if ( isset( $features ) ) {

            foreach ( $features as $feature ) {
                
                if ( $feature->name == $feature_name ) {
                    $feature_enabled = $feature->enabled;
                }
            }
        }
            
        return $feature_enabled;
    }
	
	/**
	 * Removes transient cache to be able to override shipping cost
	 * 
	 * @since 3.0.0
	 */
    public function initalize() {

        global $wpdb;
        
		$transients = $wpdb->get_col( "SELECT `option_name` FROM `".$wpdb->options."` WHERE `option_name` LIKE '_transient_wc_ship%'" );
		if ( count($transients) ) {
			foreach ( $transients as $tr ) {
				delete_transient( substr( $tr, 11 ) );
			}
        }
        
        $transient_value = get_transient( 'shipping-transient-version' );
        
        \WC_Cache_Helper::delete_version_transients( $transient_value );
        
		if ( WC()->session ) {
			WC()->session->set( 'shipping_for_package', '' );
		}
    }

	/**
	 * Load all supporting child classes
	 * 
	 * @since 3.0.0
	 */
    public function load_children() {

		//load label class (everything to do with shipping labels and printing)
        $this->label = new Shipping\Label( $this->loader );
		
		//load product class (everything to do with shipping and products)
		$this->product = new Shipping\Product( $this->loader );

		//load order class (everything to do with shipping and orders)
		$this->order = new Shipping\Order( $this->loader );
    }

	/**
	 * Manually add all different shipping methods to WooCommerce
	 * 
	 * @since 3.0.0
	 */
    public function add_shipping_methods( $methods ) {

		foreach ( $this->shipping_methods as $shipping_method ) {

			if ( get_option( $shipping_method["option"] ) === "yes" ) {
				$methods[$shipping_method["method"]] = $shipping_method["class"];
			}
		}

		return $methods;
    }
	
	/**
	 * Clears shipping rates cache, needed for calculation shipping cost
	 * 
	 * @since 3.0.0
	 */
    public function clear_shipping_rates_cache() {

        $packages = WC()->cart->get_shipping_packages();
        
		foreach ( $packages as $key => $value ) {

			$shipping_session = "shipping_for_package_$key";
			unset( WC()->session->$shipping_session );
		}
    }
	
	/**
	 * Changes order view WHERE clause to include orders using MakeCommerce shipping
	 * 
	 * @since 3.0.0
	 */
    public function shipping_filter( $where, $wp_query ) {

        global $pagenow, $wpdb;
        
        $method = !empty( $_REQUEST['_shipping_method'] ) ? $_REQUEST['_shipping_method'] : false;
        
		if ( is_admin() && $pagenow=='edit.php' && $wp_query->query_vars['post_type'] == 'shop_order' && !empty( $method ) ) {
			$where .= $GLOBALS['wpdb']->prepare( ' AND ID
				IN (
				SELECT items.order_id
				FROM '.$wpdb->prefix.'woocommerce_order_itemmeta meta, '.$wpdb->prefix.'woocommerce_order_items items
				WHERE meta.order_item_id = items.order_item_id
				AND meta.meta_key = "method_id"
				AND meta.meta_value = %s
			) ', $method );
        }
        
		return $where;
    }
	
	/**
	 * Set order parcel machine meta
	 * 
	 * @since 3.0.0
	 */
    public function set_parcel_machine_meta( $post_id ) {

		$update = false;

		$post_type = get_post_type( $post_id );

		if ( 'shop_order' !== $post_type ) {
			return;
		}

		if ( isset( $_POST['_mk_machine_id'] ) ) {

			//if the machine id is not the same as it was before then delete the _parcel_machine_shipment_id so a new one can be created.
			$current_parcel_machine = get_post_meta( $post_id, '_parcel_machine', true );

			// LP Express set order template
			if ( isset( $_POST['_mk_template_id'] ) ) {
				$current_template = get_post_meta( $post_id, '_mk_parcel_template', true );
				//set the template for the order
				if ( $_POST['_mk_template_id'] != $current_template ) {
					update_post_meta( $post_id, '_mk_parcel_template', $_POST['_mk_template_id'] );
					$update = true;
				}
			}

			if ( $_POST['_mk_machine_id'] != $current_parcel_machine || $update ) {

				//set new parcel machine
				update_post_meta( $post_id, '_parcel_machine', $_POST['_mk_machine_id'] );

				//remove old shipment id
				update_post_meta( $post_id, '_parcel_machine_shipment_id', "" );

				//register shipment again
				$this->register_shipment( $post_id );

				//update address
				self::update_order_parcelmachine_meta( $_POST['_mk_machine_id'], $post_id );
			}
		}
    }

	/**
	 * Set order parcel machine meta
	 * 
	 * @since 3.1.0
	 */
    public function update_courier_meta ( $post_id ) {

		$post_type = get_post_type( $post_id );

		if ( 'shop_order' !== $post_type ) {
			return;
		}

		// Check if a shipping method exists
		if ( isset( $_POST["shipping_method"] ) ) {
			$method = $_POST["shipping_method"];
		} else {
			return;
		}

		if ( is_array( $method ) && count( $method ) > 0 ) {
			// Check if it is a courier
			if ( array_values( $method )[0] == "courier_smartpost" || array_values( $method )[0] == "courier_omniva") {

				//remove old shipment id
				update_post_meta( $post_id, '_parcel_machine_shipment_id', "" );

				//register shipment again
				$this->register_shipment( $post_id );
			}
		}
	}

	/**
	 * Update parcel machine meta data for an order
	 * Adds all the shipping information possible.
	 * 
	 * @since 3.0.3
	 */
	public static function update_order_parcelmachine_meta( $machine_id, $order_id ) {
		
		list( $carrier, $machine ) = explode( '||', $machine_id );

		$machine = self::mk_get_machine( $carrier, $machine );
		
		if ( !empty( $machine['id'] ) ) {

			if ( empty( get_post_meta( $order_id, "_shipping_first_name", true ) ) ) {
				update_post_meta( $order_id, '_shipping_first_name', get_post_meta( $order_id, '_billing_first_name', true ) );
			}

			if ( empty( get_post_meta( $order_id, "_shipping_last_name", true ) ) ) {
				update_post_meta( $order_id, '_shipping_last_name', get_post_meta( $order_id, '_billing_last_name', true ) );
			}
			
			
			update_post_meta( $order_id, '_shipping_address_1', sanitize_text_field( $machine['name'] ) );
			update_post_meta( $order_id, '_shipping_address_2', sanitize_text_field( $machine['address'] ) );
			update_post_meta( $order_id, '_shipping_city', sanitize_text_field( $machine['city'] ) );

			update_post_meta( $order_id, '_shipping_postcode', sanitize_text_field( $machine['zip'] ) );
			update_post_meta( $order_id, '_parcel_machine', sanitize_text_field( $machine_id ) );
		}
	}

	/**
	 * Registers shipment(s) via MakeCommerce API
	 * 
	 * @since 3.0.0
	 */
    public function register_shipment( $post_ids, $select_carrier = null ) {

		//we always expect an array of post ids
		if ( !is_array( $post_ids ) ) {
			$post_ids = array( $post_ids );
		}

		//initialize to delete transient and set WC session
		$this->initalize();

		$shipping_request = array( 'credentials' => array(), 'orders' => array() );

		//setup shipping classes map
		$shipping_classes_map = array();
		foreach ( $this->shipping_methods as $shipping_method ) {

			$shipping_classes_map[$shipping_method["method"]]  = $shipping_method["class"];
		}

		//loop all post ids
		foreach ( $post_ids as $post_id ) {

			//if parcel machine shipment id already exists then skip
			$oldId = get_post_meta( $post_id, '_parcel_machine_shipment_id', true );

			if ( strlen( $oldId ) > 6 ) {
				continue;
			}

			$order = new \WC_Order( $post_id );

			//check if the post state is even paid, if not, ignore this post...
			//this could perhaps be in a way better location. Why is the function even called when post state is set to failed.
			if ( ( string )$order->get_status() !== "completed" && ( string )$order->get_status() !== "processing" ) {
				continue;
			}
			
			//skip if the order doesnt have a shipping method
			$shipping_methods = $order->get_shipping_methods();
			if ( empty( $shipping_methods ) ) {
				continue;
			}

			//loop all MakeCommerce shipping methods
			foreach ( $shipping_methods as $shipping_method ) {

				$shipping_id = explode( ':', $shipping_method['method_id'] );
				$shipping_class = $shipping_id[0];
				$shipping_instance = !empty( $shipping_id[1] ) ? $shipping_id[1] : null;
				
				//if chosen method is not the current method then move on to the next one
				if ( empty( $shipping_classes_map[$shipping_class] ) ) {
					continue;
				}
				$transport_class = new $shipping_classes_map[$shipping_class]( $shipping_instance );
				$carrier_uc = mb_strtoupper( $transport_class->carrier_id );
				
				if ( $transport_class->type === 'apt' ) {
					
					$parcel_machine = get_post_meta( $post_id, '_parcel_machine', true );
					
					if ( !$parcel_machine ) {
						continue;
					}

					list( $carrier, $machine_id ) = explode( '||', $parcel_machine );

					if ( !$carrier || !$machine_id ) {
						continue;
					}
				}

				if ( empty( $shipping_request['credentials'][$carrier_uc] ) ) {

					$api_user = $transport_class->settings['service_user'];
					$api_password = $transport_class->settings['service_password'];

					//change carrier_uc if it has been set in the interface.
					if ( isset( $transport_class->settings['service_carrier'] ) ) {
						$carrier_uc = $transport_class->settings['service_carrier'];
					}

					$use_mk_contract = false;
					if ( isset( $transport_class->settings['use_mk_contract'] ) ) {
						$use_mk_contract = $transport_class->settings['use_mk_contract'];
					}

					if ( ( !$api_user || !$api_password ) and ( !$use_mk_contract ) ) {
						add_action( 'admin_notices', 'mk_admin_error' );

						continue;
					}

					// if $use_mk_contract is enabled, do not add credentials array to the request
					if ( !$use_mk_contract ) {
						$shipping_request['credentials'][$carrier_uc] = array( 'carrier' => $carrier_uc, 'username' => $api_user, 'password' => $api_password );
					}
				}

				$sender = array(	
					'name' => !empty( $transport_class->settings['shop_name'] ) ? $transport_class->settings['shop_name'] : '',
					'phone' => !empty( $transport_class->settings['shop_phone'] ) ? $transport_class->settings['shop_phone'] : '',
					'email' => !empty( $transport_class->settings['shop_email'] ) ? $transport_class->settings['shop_email'] : '',
					'country' => !empty( $transport_class->settings['shop_address_country'] ) ? $transport_class->settings['shop_address_country'] : '',
					'city' => !empty( $transport_class->settings['shop_address_city'] ) ? $transport_class->settings['shop_address_city'] : '',
					'street' => !empty( $transport_class->settings['shop_address_street'] ) ? $transport_class->settings['shop_address_street'] : '',
					'postalCode' => !empty( $transport_class->settings['shop_postal_code'] ) ? $transport_class->settings['shop_postal_code'] : '',
					// LP Express additions
					'building' => !empty( $transport_class->settings['shop_building'] ) ? $transport_class->settings['shop_building'] : '',
					'apartment' => !empty( $transport_class->settings['shop_apartment'] ) ? $transport_class->settings['shop_apartment'] : ''
				);

				$shipping_information = $this->get_order_shipping_information( $order );

				$sr_order = array(
					'carrier' => $carrier_uc,
					'orderId' => $order->get_id(),
					'recipient' => array( 'name' => $shipping_information['recipient_name'], 'phone' => $shipping_information['phone'], 'email' => $shipping_information['email'] ),
					'sender' => $sender,
					'destination' => array(),
				);

				$m_service_type = null;
				if ( $transport_class->carrier === 'omniva' && $transport_class->type === 'atp' ) {
					$m_service_type = 'PA';
				} else if ( !empty( $transport_class->settings['registerOnPaymentCode'] ) ) {
					$m_service_type = $transport_class->settings['registerOnPaymentCode'];
				}

				if ( $transport_class->type === 'cou' || $carrier_uc === 'LP_EXPRESS_LT' ) {

					$s_postcode = method_exists( $order, 'get_shipping_postcode' ) ? $order->get_shipping_postcode() : $order->shipping_postcode;
					$s_country = method_exists( $order, 'get_shipping_country' ) ? $order->get_shipping_country() : $order->shipping_country;
					$s_state = method_exists( $order, 'get_shipping_state' ) ? $order->get_shipping_state() : $order->shipping_state;
					$s_city = method_exists( $order, 'get_shipping_city' ) ? $order->get_shipping_city() : $order->shipping_city;
					$s_address_1 = method_exists( $order, 'get_shipping_address_1' ) ? $order->get_shipping_address_1() : $order->shipping_address_1;
					$s_address_2 = method_exists( $order, 'get_shipping_address_2' ) ? $order->get_shipping_address_2() : $order->shipping_address_2;

					$sr_order['destination'] = array(
						'postalCode' => $s_postcode,
						'country' => $s_country,
						'county' => $s_state,
						'city' => $s_city,
						'street' => $s_address_1 . ' ' . $s_address_2
					);

					if ( $shipping_class == "courier_smartpost" ) {

						$delivery_time = get_post_meta( $post_id, '_delivery_time', true );
						
						if ( $delivery_time ) {
							$sr_order['destination']['timeWindow'] = $delivery_time;
						}
					}
				}

				if ( !empty( $machine_id ) ) {
					$sr_order['destination']['destinationId'] = $machine_id;
				}

				if ( $m_service_type ) {
					$sr_order['services'] = array( 'serviceType' => $m_service_type );
				}

				if ( $carrier_uc === 'LP_EXPRESS_LT' ) {
					$default = get_option( 'mk_lpexpress_template' );
					if ( !empty( get_post_meta( $post_id, '_mk_parcel_template' ) ) ) {
						$template = get_post_meta( $post_id, '_mk_parcel_template' )[0];
					} else {
						$template = $default;
					} 
					$sr_order['lpExpressShipmentDetails']["templateId"] = $template;
				}

				$shipping_request['orders'][] = $sr_order;
			}
		}
		
		$shipping_request['credentials'] = array_values( $shipping_request['credentials'] );

		if ( empty( $shipping_request['orders'] ) ) {
			return;
		}
		
		$MK = \MakeCommerce::get_api();
		if ( !$MK ) {
			return;
		}
		
		try {
			$response = $MK->createShipments( $shipping_request );
		} catch ( \Exception $e ) {
			error_log( 'Error while creating shipment ['.$e->getMessage().']' );
			return;
		}

		$shipments = !empty( $response->shipments ) ? $response->shipments : $response;
		$manifest = !empty( $response->manifests ) ? $response->manifests[0] : false;
		
		foreach ( $shipments as $order ) {
			if ( $manifest ) {
				update_post_meta( ( int )$order->orderId, '_parcel_machine_manifest', sanitize_text_field( $manifest ) );
			}
			
			if ( isset( $order->lpExpressCartIdentifier ) && !empty( $order->lpExpressCartIdentifier ) ) {
				update_post_meta( ( int )$order->orderId, '_lp_express_cart_identifier', sanitize_text_field( $order->lpExpressCartIdentifier ) );
			}

			if ( !empty( $order->orderId ) && !empty( $order->shipmentId ) ) {

				update_post_meta( ( int )$order->orderId, '_parcel_machine_shipment_id', sanitize_text_field( $order->shipmentId ) );
				update_post_meta( ( int )$order->orderId, '_tracking_number', sanitize_text_field( $order->shipmentId ) ); //default value used by other plugins, such as WooCommerce PDF Invoices and so on
				delete_post_meta( ( int )$order->orderId, '_parcel_machine_error' );
			} else if ( !empty( $order->orderId ) && !empty( $order->barCode ) ) {

				update_post_meta( ( int )$order->orderId, '_parcel_machine_shipment_id', sanitize_text_field( $order->barCode ) );
				update_post_meta( ( int )$order->orderId, '_tracking_number', sanitize_text_field( $order->barCode ) ); //default value used by other plugins, such as WooCommerce PDF Invoices and so on
				delete_post_meta( ( int )$order->orderId, '_parcel_machine_error' );
			} else if ( !empty( $order->orderId ) && !empty( $order->errorMessage ) ) {

				update_post_meta( ( int )$order->orderId, '_parcel_machine_error', sanitize_text_field( $order->errorMessage ) );
			}
		}
	}

	/**
	 * Returns shipping information for order
	 * 
	 * @since 3.0.9
	 */
	public function get_order_shipping_information( $order) {

		$shipping_information = [];

		$shipping_address = $order->get_address( 'shipping' );
		$billing_address = $order->get_address( 'billing' );

		$phone = '';
		if ( isset( $shipping_address['phone'] ) ) {
			$phone = $shipping_address['phone'];
		}
		
		if ( $phone == '' && isset( $billing_address['phone'] ) ) {
			$phone = $billing_address['phone'];
		}

		$shipping_information['first_name'] = $shipping_address['first_name'] ? $shipping_address['first_name'] : $billing_address['first_name'];
		$shipping_information['last_name'] = $shipping_address['last_name'] ? $shipping_address['last_name'] : $billing_address['last_name'];
		$shipping_information['phone'] = $phone;
		$shipping_information['email'] = get_post_meta( $order->get_id(), '_shipping_email', true ) ? get_post_meta( $order->get_id(), '_shipping_email', true ) : $billing_address['email'];

		$shipping_information['recipient_name'] = $shipping_information['first_name'] . ' ' . $shipping_information['last_name'];

		return $shipping_information;
	}

    /**
	 * Returns the correct link to be used for tracking link
	 * 
	 * @since 3.0.0
	 */
	public function get_tracking_link( $carrier, $order_id, $shopLocation = false ) {

		//do we use shop location or delivery location
		if ( $shopLocation ) {
			//get_base_country returns either EE, LT or LV. Everything else is irrelevant
			$tld = substr( strtolower( WC()->countries->get_base_country() ), 0, 2);

			$userLang = $tld;
		} else {

			//get order delivery location. Returns EE, LT or LV. Evertyhing else is irrelevant
			$tld = substr( strtolower( get_post_meta( $order_id, '_shipping_country', true ) ), 0, 2 );
			$userLang = substr( strtolower( get_post_meta( $order_id, 'wpml_language', true ) ), 0, 2 ); //returns nothing if it doesnt exist, otherwise returns en, et, lt, lv, ru
		}
		
		//
		//  create array for different options [carrier][tld][lang]
		//	first one in the list must be default, this goes for the tld as well as for the userLang.
		//

		/* Omniva */
		//EST
		$trackingURLS["omniva"]["ee"]["ee"] = 'https://www.omniva.ee/abi/jalgimine?barcode=';
		$trackingURLS["omniva"]["ee"]["en"] = 'https://www.omniva.ee/private/track_and_trace?barcode=';
		$trackingURLS["omniva"]["ee"]["ru"] = 'https://www.omniva.ee/chastnyj/otslezhivanie_posylki?barcode=';

		//LT
		$trackingURLS["omniva"]["lt"]["lt"] = 'https://www.omniva.lt/verslo/siuntos_sekimas?barcode=';
		$trackingURLS["omniva"]["lt"]["en"] = 'https://www.omniva.lt/business/track_and_trace?barcode=';

		//LV
		$trackingURLS["omniva"]["lv"]["lv"] = 'https://www.omniva.lv/privats/sutijuma_atrasanas_vieta?barcode=';
		$trackingURLS["omniva"]["lv"]["en"] = 'https://www.omniva.lv/private/track_and_trace?barcode=';
		$trackingURLS["omniva"]["lv"]["ru"] = 'https://www.omniva.lv/chastnyj/mestonahozhdenie_posylki?barcode=';

		/* DPD */
		//EE
		$trackingURLS["dpd"]["ee"]["ee"] = 'https://tracking.dpd.de/status/et_EE/parcel/';
		$trackingURLS["dpd"]["ee"]["en"] = 'https://tracking.dpd.de/status/en_EE/parcel/';

		//LT
		$trackingURLS["dpd"]["lt"]["lt"] = 'https://tracking.dpd.de/status/lt_LT/parcel/';
		$trackingURLS["dpd"]["lt"]["en"] = 'https://tracking.dpd.de/status/en_LT/parcel/';

		//LV
		$trackingURLS["dpd"]["lv"]["lv"] = 'https://tracking.dpd.de/status/lv_LV/parcel/';
		$trackingURLS["dpd"]["lv"]["en"] = 'https://tracking.dpd.de/status/en_LV/parcel/';

		/* Smartpost */
		//EE
		$trackingURLS["smartpost"]["ee"]["ee"] = 'https://itella.ee/eraklient/saadetise-jalgimine/?trackingCode=';
		$trackingURLS["smartpost"]["ee"]["en"] = 'https://itella.ee/en/private-customer/parcel-tracking/?trackingCode=';
		$trackingURLS["smartpost"]["ee"]["ru"] = 'https://itella.ee/ru/chastnyj-klijent/otslezhivanije-otpravlenija/?trackingCode=';

		//FI
		$trackingURLS["smartpost"]["fi"]["en"] = 'https://www.posti.fi/en/tracking#/lahetys/';
		$trackingURLS["smartpost"]["fi"]["fi"] = 'https://www.posti.fi/fi/seuranta#/lahetys/';
		$trackingURLS["smartpost"]["fi"]["sv"] = 'https://www.posti.fi/sv/uppfoljning#/lahetys/';

		//LT
		$trackingURLS["smartpost"]["lt"]["lt"] = 'https://itella.lt/private-customer/krovinio-sekimas/?trackingCode=';
		$trackingURLS["smartpost"]["lt"]["en"] = 'https://itella.lt/en/private-customer/track-shipment/?trackingCode=';

		//LV
		$trackingURLS["smartpost"]["lv"]["lv"] = 'https://itella.lv/private-customer/sutijuma-meklesana/?trackingCode=';
		$trackingURLS["smartpost"]["lv"]["en"] = 'https://itella.lv/en/private-customer/parcel-tracking/?trackingCode=';

		//carrier exists
		if ( isset( $trackingURLS[$carrier] ) ) {
			//country exists
			if ( isset( $trackingURLS[$carrier][$tld] ) ) {
				if ( isset( $trackingURLS[$carrier][$tld][$userLang] ) ) {
					return $trackingURLS[$carrier][$tld][$userLang];
				} else {
					//selected language not found, return carrier tld default language.
					return current( $trackingURLS[$carrier][$tld] );
				}
			} else {
				// selected tld not found, return tld based on store country
				$tld = substr( strtolower( WC()->countries->get_base_country() ), 0, 2);
				if ( isset( $trackingURLS[$carrier][$tld] ) ) {
					$defaultCarrierTLD = current( $trackingURLS[$carrier][$tld] );
				} else {
					// no store country was found either
					$defaultCarrierTLD = current( $trackingURLS[$carrier] );
				}

				return $defaultCarrierTLD;
			}
		}

		//not even carrier was found. Return nothing
		return "";
	}
	
	/**
	 * Returns all available parcelmachines
	 * 
	 * @since 3.0.0
	 */
	public static function mk_get_machines( $carrier, $country = null, $aptopts = [] ) {

		//set machine cache
		$data = get_option( 'mk_machines_cache', false );
		$data_expires = get_option( 'mk_machines_expires', false );

		if ( !$data || empty( $data ) || $data_expires < time() ) {

			$MK = \MakeCommerce::get_api();
			
			if( !$MK ) {
				return array();
			}

			$data = $MK->getDestinations( array( 'type' => 'APT,PUP' ) );
			
			update_option( 'mk_machines_cache', $data, 'no' );
			update_option( 'mk_machines_expires', time() + 3 * 60 * 60, 'no' );
		}
		
		//return empty if no machines found for destination
		if ( !$data || empty( $data ) ) {
			
			return array();
		}
		
		//create machines array
		$machines = array();

		foreach ( $data as $machine ) {

			if ( ( !$country || $machine->country === $country ) && 
				 ( $machine->type === 'APT' || $machine->type === 'PUP' ) && 
				 ( $carrier === '*' || strtolower( $carrier ) === strtolower( $machine->carrier ) ) ) {
				
				if ( !(isset($aptopts['use_white_apts']) && $aptopts['use_white_apts'] == 1 &&
				     $machine->carrier == 'SMARTPOST' && strpos($machine->name, 'valge') != false)) {
					$machines[] = array(
						'carrier' => strtolower( $machine->carrier ),
						'id' => $machine->id,
						'name' => $machine->name, 
						'city' => $machine->city,
						'address' => !empty( $machine->address ) ? $machine->address : '',
						'zip' => $machine->zip
					);
				}
			}
		}

		//sort machines
		usort( $machines, function( $a, $b ) { 
			if ( $a['city'] === $b['city'] ) {
				
				return $a['name'] > $b['name'] ? 1 : -1;
			}

			return $a['city'] > $b['city'] ? 1 : -1;
		});

		return $machines;
	}

	/**
	 * Returns a specific parecelmachine
	 * 
	 * @since 3.0.0
	 */
	public static function mk_get_machine( $carrier, $id ) {

		//get all available machines
		$machines = self::mk_get_machines( $carrier );

		//loop and return the machine we are searching for if found
		foreach ( $machines as $machine ) {

			if ( $machine['carrier'] === $carrier && $machine['id'] == $id ) {

				return $machine;
			}
		}

		return false;
	}
    
	/**
	 * Register the stylesheets.
	 *
	 * @since 3.0.0
	 */
	public function enqueue_styles() {

		if ( is_admin() ) {
			wp_enqueue_style( $this->plugin_name . "-parcelmachine-admin", plugin_dir_url(__FILE__) . 'css/parcelmachine-admin.css', array(), $this->version );
		}
	}

	/**
	 * Register the JavaScript.
	 *
	 * @since 3.0.0
	 */
	public function enqueue_scripts() {

		if ( is_admin() ) {
			\MakeCommerce::mc_enqueue_script('MC_PARCELMACHINE_JS_ADMIN', dirname(__FILE__) . '/js/parcelmachine-admin.js', [], [ 'jquery' ]);
		} else {

			\MakeCommerce::mc_enqueue_script('MC_PARCELMACHINE_SEARCHABLE_JS', dirname(__FILE__) . '/js/parcelmachine_searchable.js', 
				[ array( 'placeholder' => __( '-- select parcel machine --', 'wc_makecommerce_domain' ) ) ], [ 'jquery' ]
			);

			\MakeCommerce::mc_enqueue_script('MC_PARCELMACHINE_JS', dirname(__FILE__) . '/js/parcelmachine.js', [], [ 'jquery' ]);
		}
	}
}
